// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: Esizeit
// Author: Richard Saxton 
//
// Description: Main class for calculating optimum image region
//
// Modifications: See ChangeLog
//

#ifndef Esizeit_H
#define Esizeit_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <Xmm.h>
#include "CalServer.h"
#include "STime.h"

/* Local Classes */
#include "ArrayPixelIterator.h"
#include "WcsCC.h"

//extern F90MODVARDEF(WcsCC, especget_glue, wcs);

/**************************************************n
 *
 * class Esizeit
 *
 ***************************************************/
class Esizeit
{
public:

  /* Constructor */
  Esizeit();

  /* Destructor */
  ~Esizeit();

  struct ImageData {
      DataSet * imageset;
      string instrument;
      string obsmode;
      double exposure;
      ArrayPixelIterator *image;
  };

  /* Reads the image and sets the ImageData structure */
  ImageData readImage(const string & imagefile);

  /* Find the exposure value at a given celestial position */
  void handleExposure(string & expfile);
  double getExposure(const string & exposurefile, double src_ra,
                                    double src_dec, double defaultExp);


private:

  /* Member fields */
  vector<pair<int, int> > _megapix;

};

#endif

