// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: Eupper
// Author: Richard Saxton 
//
// Description: Main class for calculating optimum image region
//
// Modifications: See ChangeLog
//

#ifndef Eupper_H
#define Eupper_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <Xmm.h>
#include "CalServer.h"
#include "STime.h"

/* Local Classes */
#include "ArrayPixelIterator.h"
#include "WcsCC.h"

//extern F90MODVARDEF(WcsCC, especget_glue, wcs);

/**************************************************n
 *
 * class Eupper
 *
 ***************************************************/
/*
struct WcsAxisTcc
{
    char ctype[8],           // CTYPE
      worldUnit[8];         // CUNIT
    double matrixOnAxis,   // CD1_
      matrixOffAxis,   // CD2_
      refPixels,       // CRPIX
      refWorld;          // CRVAL
};

struct wcscc
{
   WcsAxisTcc x,y;
   //bool crotFound;
   int crotFound;  // Actually Logical but think that it is 4 bytes.
};
*/

class Eupper
{
public:

  /* Constructor */
  Eupper();

  /* Destructor */
  ~Eupper();

  /* Define data structure */
  struct COUNTS {
      double counts,              // Total counts in extraction area
             area;                // Number of pixels used to extract counts
      //       bkgsub_count_rate,          // Bckgnd subtracted count rate
        //     bkgsub_count_rate_error,    // Error on subtracted count rate
          //   ee,                         // Applied encircled energy correction
            // upplim;                     // Upper limit on source counts
  };

  struct ImageData {
      DataSet * imageset;
      string instrument;
      string obsmode;
      double exposure;
      ArrayPixelIterator *image;
      string obsid;
      string start_date;
      string end_date;

  };

  /* Calculate the upper limit */
  double calcUpplim(double n, double b, double cl);

  /* Reads the image and sets the ImageData structure */
  ImageData readImage(const string & imagefile);

  /* Calculates the counts in an image region */
  COUNTS counts(ImageData imdat, double ra, double dec,
                      double rad_inner, double rad_outer);

  /* Find the exposure value at a given celestial position */
  double getExposure(const string & exposurefile, double src_ra,
                                    double src_dec, double defaultExp);

  /* Calculate the upper limit */
  double calcUpperlimit(double exposure, double src_counts,
                        double bck_counts, double toterr,
                        double cl, double eef);

  /* Get the encircled energy factor */
  double getEEF(ImageData srcImage, double src_rad) const;

  /* Find if axes in right order */
  bool checkaxes(Array *array);

  /* Find the pixel corresponding to this celestial position */
  PixelElement findPixel(ImageData imdat, double ra, double dec);

  /* Split string into words */
  vector<string> splitString(const string & str);

private:


  /* Member fields */

};

#endif

