// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: FluxedSpectrum
// Author: Richard Saxton
//
// Description: 
// Provides an clean interface to a fluxed spectrum or to an ARF
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: FluxedSpectrum.cc,v 1.1 2013/06/06 15:57:39 rsaxton Exp $

// General classes
#include<cassert>

#include <string>
#include <iostream>
#include <vector>
#include <map>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"
#include "EnergySpectrum.h"
#include "FluxedSpectrum.h"
#include "CalOalUtility.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class FluxedSpectrum

// Supports data consisting of an energy, energy width and data value

// -----------------------------------------------------------------------------------

 /* Constructor - from a fluxed spectrum */
  FluxedSpectrum::FluxedSpectrum(const string & fluxedspectrumName)
    {
      PROC;

      // Read file
      DataSet * set   = dataSetServer->open(fluxedspectrumName, DataSet::Read);
     
      // Open the spectrum table
      Table * table = set->table("FluxedSpectrum");

      // Store the table size
      _N = table->rows();
      //cout << "No elements "<< _N << ".." <<endl;;

  /* Read data from the central energy and energy width columns
   Column ecol = table->column("ENERGY");
   Column ecolw = table->column("ENERGY_BIN");
   Column fcol = table->column("FLUX");
   ColumnData * ecoldata = ecol->data(); 
   ColumnData * ecoldataw = ecolw->data(); 
   ColumnData * fcoldata = fcol->data(); 
   real32     * energy = ecoldata->real32Data();
   real32     * energy_width = ecoldataw->real32Data();
   real32     * flux = fcoldata->real32Data();
  */
      // Read in the energy, width and flux columns
      CalOalUtility::fillVector(table, "ENERGY", _energy);
      CalOalUtility::fillVector(table, "ENERGY_BIN", _energyWidth);
      CalOalUtility::fillVector(table, "FLUX", _flux);

      // Convert flux to count rate / cm^2
      _value = convFluxCrate(_flux);

    }

/* Destructor */
FluxedSpectrum:: ~FluxedSpectrum()
{
  PROC;
  ENDPROC;
};

// Method: convFluxCrate - calculates the count rate (/cm^2) in each energy bin
//                         by dividing flux by energy of each photon
vector<float> FluxedSpectrum::convFluxCrate(vector<float> flux) const
{

     vector<float> countRate;

     // Convert from a flux to count rate by dividing by the energy of 
     // each photon - assumed to be in keV
      for (int i=0; i<_N; i++) {

        if (_energy[i] > 0.0)
          countRate.push_back(flux[i] / (_energy[i] * 1000.0 * 1.60217646E-12));
        else
          countRate.push_back(0.0);

      }

      return countRate;
}
//
// Method: countRate - returns the count rate per cm^2 in each band
//
vector<float> FluxedSpectrum::flux() const
{
      return _flux;
}



