// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// RmfMaker.cc
//
// Authors:  Richard Saxton (VEGA/ESA)
//           ESA XMM SOC
//
// Description: Runs rmfgen to produce an ARF
//
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>

#include <rmfgen.h>   // tasks

#include "RmfMaker.h"

RmfMaker::RmfMaker(string spectrumset, string rmfset,
		   bool extendedsource, 
                   bool withdetbins, unsigned xbins, unsigned ybins,
                   bool withsourcepos, string sourcecoords,
                   float sourcex, float sourcey,
		   bool withenergybins, float emin,float emax, 
                   int nbins, bool acceptchanrange)
{
  PROC;

  /* Set up the parameters for calling rmfgen */
    openParameters("rmfgen");

  /* First the input parameters */
    setParameter("spectrumset",spectrumset);
    setParameter("rmfset",rmfset);

  /* Set the other parameters */

    setParameter("extendedsource",extendedsource);
                                                                                
    if (extendedsource)
    {
       setParameter("detmaptype","flat");
    }
    else{
       setParameter("detmaptype","psf");
    }

    setParameter("withsourcepos",withsourcepos);
    setParameter("sourcecoords",sourcecoords);
    setParameter("sourcex",sourcex);
    setParameter("sourcey",sourcey);

  /* Set the energy bin defaults explicitly here if necessary */
    setParameter("withenergybins",withenergybins);    
    if (withenergybins) {
       setParameter("energymin",emin );
       setParameter("energymax",emax );
       setParameter("nenergybins",nbins);
    } else { 
       setParameter("energymin",0.05);
       setParameter("energymax",12.0);
       setParameter("nenergybins",120);
    }

    setParameter("acceptchanrange",acceptchanrange);

  /* Now run rmfgen using these parameters */ 
    task(rmfgen);
   
  /* Shutdown */
    closeParameters();

  ENDPROC;
}

RmfMaker::~RmfMaker()
{
  PROC;
  ENDPROC;
}

