// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: SpectrumMaker
// Author: Richard Saxton 
//
// Description: Creates a spectrum using evselect
//
// Modifications: See ChangeLog
//

#ifndef SpectrumMaker_H
#define SpectrumMaker_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <Xmm.h>
#include "DatafileAttributes.h"

/* Local Classes */

/****************************************************
 *
 * class SpectrumMaker
 *
 ***************************************************/

class SpectrumMaker
{
public:

  /* Constructor */
  SpectrumMaker(string spectrumfile, DatafileAttributes * specAttributes, 
                                                         string expression,bool multiEPICspec);

  /* Destructor */
  ~SpectrumMaker();

  /* Method to add keywords to spectrum */
  void addkeywords(string bgdfile,
                                string rspfile, string arffile);

private:


  /* Member fields */

  Xmm::Instrument       _instrumentID;
  string               _instr;
  int                  _specbin;
  double	       _specmin;
  double               _specmax;
  string               _spectrumfile;
  string               _flagexp;
  string               _pattexp;

};

#endif

