// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Module: UpperLimitSolver
// Author: Richard Saxton 
//
// Description: Creates an ARF using arfgen
//
// Modifications: See ChangeLog
//
#ifndef UpperLimitSolver_H
#define UpperLimitSolver_H

/* General classes */
#include <iostream>
#include<cmath>
#include<vector>
#include<utility>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>

/* Local Classes */

/****************************************************
 *
 * class UpperLimitSolver
 *
 ***************************************************/

class UpperLimitSolver
{
public:

  /* Constructor */
  UpperLimitSolver(long int n, double b);

  /* Destructor */
  ~UpperLimitSolver();

  /* The bayesian constant */
  void Const_C();

  /* The Bayesian calculation */
  std::pair<double, double> bayesian(const double CL);

  /* Calculation of factorial */
  double factorial(const double n);

private:

  /* The Bayesian calculation */
  std::pair<double, double> Bayesian_Confidence_Limits(
                                                     const double CL);

  /* Calculates the Bayesian posterior probability function */
  double ppdf(const double S);

  double B;
  long int N;

  double C;

};

#endif

